var express = require('express');
var path = require('path');
//var favicon = require('serve-favicon');
var logger = require('morgan');
var cookieParser = require('cookie-parser');
var bodyParser = require('body-parser');
var session = require('express-session');
var MemoryStore = require('memorystore')(session);
var flash = require('connect-flash');
var app = express();
// favicon stuff
//app.use(favicon(path.join(__dirname, 'favicons', 'favicon.ico')));

// view engine setup
app.set('views', path.join(__dirname, './views'));
app.set('view engine', 'pug');
app.use(logger('dev'));

app.use(bodyParser.json({ limit: '50mb' }));
app.use(
	bodyParser.urlencoded({
		extended: false,
		limit: '50mb'
	})
);
app.use(cookieParser());
app.use(express.static(path.join(__dirname, '../public'), { extensions: ['html'] }));

app.use(
	session({
		store: new MemoryStore({
			checkPeriod: 86400000 // prune expired entries every 24h
		}),
		secret: '1KqZ18W8KskE1iSw',
		saveUninitialized: false,
		resave: false,
		cookie: {
			maxAge: 608800000
		}
	})
);
app.use(flash());
//sections
//var front = require('./routes/front/index')(session);
var dash = require('./routes/dash/index');
var page = require('./routes/dash/pages');
var settings = require('./routes/dash/settings');
var nav = require('./routes/dash/nav');
//api
var pages = require('./api/v1/pages');
var setting = require('./api/v1/settings');
var mailer = require('./api/v1/mailer');
var auth = require('./api/v1/auth');
var backup = require('./api/v1/backup');
// API PATHS

app.use('/api/v1/page', pages);
app.use('/api/v1/settings', setting);
app.use('/api/v1/auth', auth);
app.use('/api/v1/mailer', mailer);
app.use('/api/v1/backup', backup);
// PAGES
app.use('/@/dashboard', dash);
app.use('/@/dashboard/page', page);
app.use('/@/dashboard/settings', settings);
app.use('/@/dashboard/navigation', nav);
// catch 404 and forward to error handler
app.use(function (req, res, next) {
	var err = new Error('Not Found');
	err.status = 404;
	next(err);
});
// error handlers
// development error handler
// will print stacktrace
if (app.get('env') === 'development') {
	app.use(function (err, req, res) {
		res.status(err.status || 500);
		res.render('error', {
			message: err.message,
			error: err
		});
	});
}
// production error handler
// no stacktraces leaked to user
app.use(function (err, req, res) {
	res.status(err.status || 500);
	res.render('error', {
		message: err.message,
		error: {}
	});
});
module.exports = app;
